@@ -5,11 +5,14 @@
 # Once you have melted your data, cast will arrange it into the form you desire
 # based on the specification given by \code{formula}.
 #
-# The cast formula has the following format: \code{row_variable_1 + row_2 ~ col_variable + col_2}
+# The cast formula has the following format: \code{x_variable + x_2 ~ y_variable + y_2 ~ z_variable ~  ... | list_variable + ... }
 # The order of the variables makes a difference.  The first varies slowest, and the last 
 # fastest.  There are a couple of special variables: "..." represents all other variables 
 # not used in the formula and "." represents no variable, so you can do \code{formula=var1 ~ .}
 #
+# Creating high-D arrays is simple, and allows a class of transformations that are hard
+# without \code{\link{iapply}} and \code{\link{sweep}} 
+#
 # If the combination of variables you supply does not uniquely identify one row in the 
 # original data set, you will need to supply an aggregating function, \code{fun.aggregate}.
 # This function should take a vector of numbers and return a summary statistic(s).  It must
@@ -35,16 +38,19 @@
 # @arguments further arguments are passed to aggregating function
 # @arguments vector of variable names (can include "grand\_col" and "grand\_row") to compute margins for, or TRUE to computer all margins
 # @arguments logical vector to subset data set with before reshaping
-# @seealso \code{\link{reshape1}}, \url{http://had.co.nz/reshape/}
+# @seealso \code{\link{reshape1}}, vignette("introduction", "reshape"), \url{http://had.co.nz/reshape/}
 #X #Air quality example
 #X names(airquality) <- tolower(names(airquality))
 #X airquality.d <- melt(airquality, id=c("month", "day"), preserve.na=FALSE)
 #X 
+#X cast(airquality.d, day ~ month ~ variable)
 #X cast(airquality.d, month ~ variable, mean)
+#X cast(airquality.d, month ~ . | variable, mean)
 #X cast(airquality.d, month ~ variable, mean, margins=c("grand_row", "grand_col"))
 #X cast(airquality.d, day ~ month, mean, subset=variable=="ozone")
 #X cast(airquality.d, month ~ variable, range)
 #X cast(airquality.d, month ~ variable + result_variable, range)
+#X cast(airquality.d, variable ~ month ~ result_variable,range)
 #X
 #X #Chick weight example
 #X names(ChickWeight) <- tolower(names(ChickWeight))
@@ -52,6 +58,7 @@
 #X 
 #X cast(chick.d, time ~ variable, mean) # average effect of time
 #X cast(chick.d, diet ~ variable, mean) # average effect of diet
+#X cast(chick.d, diet ~ time ~ variable, mean) # average effect of diet & time
 #X 
 #X # How many chicks at each time? - checking for balance
 #X cast(chick.d, time ~ diet, length)
@@ -59,28 +66,36 @@
 #X cast(chick.d, chick ~ time, mean, subset=time < 10 & chick < 20)
 #X 
 #X cast(chick.d, diet + chick ~ time)
+#X cast(chick.d, chick ~ time ~ diet)
 #X cast(chick.d, diet + chick ~ time, mean, margins="diet")
+#X
 #X #Tips example
-#X data(tips)
 #X cast(melt(tips), sex ~ smoker, mean, subset=variable=="total_bill")
+#X cast(melt(tips), sex ~ smoker | variable, mean)
 #X 
-#X data(french_fries)
-#X ff_d <- melt(french_fries, id=1:4)
+#X ff_d <- melt(french_fries, id=1:4, preserve.na=FALSE)
 #X cast(ff_d, subject ~ time, length)
 #X cast(ff_d, subject ~ time, function(x) 30 - length(x))
 #X cast(ff_d, variable ~ ., function(x) c(min=min(x), max=max(x)))
+#X cast(ff_d, variable ~ ., function(x) quantile(x,c(0.25,0.5)))
 #X cast(ff_d, treatment ~ variable, mean, margins=c("grand_col", "grand_row"))
 #X cast(ff_d, treatment + subject ~ variable, mean, margins="treatment")
 #X lattice::xyplot(X1 ~ X2 | variable, cast(ff_d, ... ~ rep), aspect="iso")
-cast <- function(data, formula = ... ~ variable, fun.aggregate, ..., margins, subset=TRUE) {
-	dataname <- deparse(substitute(data))
+cast <- function(data, formula = ... ~ variable, fun.aggregate=NULL, ..., margins=FALSE, subset=TRUE, df=FALSE) {
+  if (!is.character(formula)) formula <- deparse(substitute(formula))
 	subset <- eval(substitute(subset), data, parent.frame())  
 	data <- data[subset, ]  
-	variables <- cast_parse_formula(deparse(substitute(formula)), names(data))
+	variables <- cast_parse_formula(formula, names(data))
 	
-	res <- reshape1(data, variables, fun.aggregate, margins=margins, ...)
-	attr(res, "formula") <- deparse(substitute(formula))
-	attr(res, "data") <- dataname
+	if (!is.null(variables$l)) {
+		res <- nested.by(data, data[variables$l], function(x) {
+			reshape1(x, variables$m, fun.aggregate, margins=margins, df=df, ...)
+		})	
+	} else {
+		res <- reshape1(data, variables$m, fun.aggregate, margins=margins, df=df, ...)
+	}
+	attr(res, "formula") <- formula
+	#attr(res, "data") <- deparse(substitute(data))
 	
 	res
 }
@@ -92,12 +107,13 @@ cast <- function(data, formula = ... ~ variable, fun.aggregate, ..., margins, su
 # @arguments variables to appear in columns
 # @arguments variables to appear in rows
 # @arguments aggregation function
+# @arguments should the aggregating function be supplied with the entire data frame, or just the relevant entries from the values column
 # @arguments vector of variable names (can include "grand\_col" and "grand\_row") to compute margins for, or TRUE to computer all margins
 # @arguments further arguments are passed to aggregating function
 # @seealso \code{\link{cast}}
 # @keyword internal
 #X names(airquality) <- tolower(names(airquality))
-#X airquality.d <- melt(airquality, id=c("month", "day"))
+#X airquality.d <- melt(airquality, id=c("month", "day"), preserve=FALSE)
 #X #Basic call
 #X reshape1(airquality.d, list("month", NULL), mean)
 #X reshape1(airquality.d, list("month", "variable"), mean)
@@ -133,70 +149,53 @@ cast <- function(data, formula = ... ~ variable, fun.aggregate, ..., margins, su
 #X reshape1(airquality.d, list(c("month", "result_variable"), "variable"), range, c("grand_row"))
 #X 
 #X reshape1(airquality.d, list(c("month"), c("variable")), function(x) diff(range(x))) 
-reshape1 <- function(data, vars = list(NULL, NULL), fun.aggregate, margins, ...) {
-	rows <- vars[[1]]
-	cols <- vars[[2]]
-
-	cols.clean <- clean.vars(cols)
-	rows.clean <- clean.vars(rows)
-	variables <- c(rows.clean, cols.clean)
+reshape1 <- function(data, vars = list(NULL, NULL), fun.aggregate=NULL, margins, df=FALSE, ...) {
+	vars.clean <- lapply(vars, clean.vars)
+	variables <- unlist(vars.clean)
 
 	if (!missing(margins) && isTRUE(margins)) margins <- c(variables, "grand_row", "grand_col")
 	
 	aggregate <- nrow(unique(data[,variables, drop=FALSE])) < nrow(data)
 	if (aggregate) {
-		if (missing(fun.aggregate)) {
+		if (missing(fun.aggregate) || is.null(fun.aggregate)) {
 			warning("Aggregation requires fun.aggregate: length used as default", call.=FALSE)
 			fun.aggregate <- length
 		}
-		data.r <- expand(condense(data, variables, fun.aggregate, ...))
+		if (!df) {
+		  data.r <- expand(condense(data, variables, fun.aggregate, ...)) 
+		} else {
+		  data.r <- condense.df(data, variables, fun.aggregate, ...)
+		}
 
-		if ("result_variable" %in% names(data.r) && !("result_variable" %in% c(rows,cols))) {
-			cols <- c(cols, "result_variable")
+		if ("result_variable" %in% names(data.r) && !("result_variable" %in% unlist(vars))) {
+			vars[[2]] <- c(vars[[2]], "result_variable")
 		}
 	} else {
 		data.r <- data.frame(data[,c(variables)], result = data$value)	
 	}
-
-	margins.r <- compute.margins(data, margin.vars(rows.clean, cols.clean, margins), fun.aggregate, ...)
-	result <- sort.df(rbind.fill(data.r, margins.r), c(rows,cols))
+  
+  if (length(vars.clean) > 2 && margins) {
+    warning("Sorry, you currently can't use margins with high D arrays", .call=FALSE)
+    margins <- FALSE
+  }
+	margins.r <- compute.margins(data, margin.vars(vars.clean, margins), fun.aggregate, ..., df=df)
+	result <- sort.df(rbind.fill(data.r, margins.r), unlist(vars))
 	result <- add.all.combinations(result, vars)
 
-	dim.names <- function(data, vars) {
-		if (!is.null(vars) && length(vars) > 0) {
-			unique(data[,vars,drop=FALSE]) 
-		} else {
-			data.frame(value="value") # use fun.aggregate instead of "value"? 
-		}
-	}
-	row.names <- dim.names(result, rows)
-	col.names <- dim.names(result, cols)
+	dimnames <- lapply(vars, function(x) dim.names(result, x))
 
-	reshaped <- matrix(unlist(result$result), nrow=nrow(row.names), ncol=nrow(col.names), byrow=TRUE)
-	
+  r <- if (!df) unlist(result$result) else result$result
+	reshaped <- array(r, rev(sapply(dimnames, nrow)))
+  
+  reshaped <- aperm(reshaped, length(dim(reshaped)):1)
+	dimnames(reshaped) <- lapply(dimnames, function(x) apply(x, 1, paste, collapse="-"))
 	rownames(reshaped) <- 1:nrow(reshaped)
 	
-	as.data.frame(cast_matrix(reshaped, list(row.names, col.names)))
+	if (length(vars.clean) > 2) return(reshaped)
+	if (df) return(cast_matrix(reshaped, dimnames))
+	as.data.frame(cast_matrix(reshaped, dimnames))
 }
 
-# Compute margins
-# Compute marginal values.
-# 
-# @arguments data frame
-# @arguments margins to compute
-# @arguments aggregation function
-# @arguments other argument passed to aggregation function
-# @keyword internal 
-compute.margins <- function(data, margins, fun.aggregate, ...) {
-	if (length(margins) == 0) return(data.frame())
-	
-	if (missing(fun.aggregate)) {
-		warning("Margins require fun.aggregate: length used as default", call.=FALSE)
-		fun.aggregate <- length
-	}
-	
-	do.call("rbind.fill",lapply(margins, function(x) expand(condense(data, x, fun.aggregate, ...))))
-}
 
 # Add all combinations
 # Add all combinations of the given rows and columns to the data frames.
@@ -207,87 +206,105 @@ compute.margins <- function(data, margins, fun.aggregate, ...) {
 # @arguments data.frame
 # @arguments variables (list of character vectors)
 # @keyword internal 
-add.all.combinations <- function(data, vars = list(NULL, NULL)) {
-	rows <- vars[[1]]
-	cols <- vars[[2]]
+#X rdunif <- 
+#X   function(n=20, min=0, max=10) floor(runif(n,min, max))
+#X df <- data.frame(a = rdunif(), b = rdunif(),c = rdunif())
+#X add.all.combinations(df)
+#X add.all.combinations(df, list("a", "b"))
+#X add.all.combinations(df, list(c("a", "b")))
+#X add.all.combinations(df, list("a", "b", "c"))
+#X add.all.combinations(df, list(c("a", "b"), "c"))
+#X add.all.combinations(df, list(c("a", "b", "c")))
+add.all.combinations <- function(data, vars = list(NULL)) {
+	if (sum(sapply(vars, length)) == 0) return(data)
 
-	if (length(rows) + length(cols) == 0) return(data)
-	all.combinations <- data.frame(expand.grid.df(data[,rows, drop=FALSE], data[,cols, drop=FALSE]))
+	all.combinations <- do.call(expand.grid.df, 
+		lapply(vars, function(cols) data[, cols, drop=FALSE])
+	)
 	result <- merge_recurse(list(data, all.combinations)) 
 	
-	sort.df(result, c(rows,cols))
+	sort.df(result, unlist(vars))
 }
 
-# cast parse formula
-# Parse formula for casting
-#
-# @value row character vector of row names
-# @value col character vector of column names
-# @value aggregate boolean whether aggregation will occur
-# @keyword internal
-cast_parse_formula <- function(formula = "...  ~ variable", varnames) {
-	check_formula(formula, varnames)
-	
-	vars <- all.vars.character(formula)
-	
-	vars <- lapply(vars, function(x) x[x != "."])
-	vars[sapply(vars, length) == 0 ] <- list(NULL)
 
-	remainder <- varnames[!(varnames %in% c(unlist(vars), "value"))]
-	
-	lapply(vars, function(x) if (any(x == "..."))  c(x[x != "..."], remainder) else x)
-}
-
-# Get all variables
-# All variables in character string of formula
+# Compute margins
+# Compute marginal values.
 # 
-# @keyword internal
-# @returns list of variables in each part of formula 
-all.vars.character <- function(formula) {
-	vars <- strsplit(formula, "~")[[1]]
-	lapply(vars, function(x) strsplit(gsub("\\s+", "", x), "[*+]")[[1]])
-}
-
-# Check formula
-# Checks that formula is a valid reshaping formula.
-#
-# \enumerate{
-#   \item expressions match known variables
-#	 \item same variable not used on both sides
-# }
-# @arguments formula to check
-# @arguments vector of variable names
-# @keyword internal
-check_formula <- function(formula, varnames) {
-	vars <- unlist(all.vars.character(formula))
-	if (!all(vars %in% c(".", "...","result_variable",varnames))) stop("Formula contains variables not in list of known variables")
-	vars <- vars[vars != "."]
-	if (length(unique(vars)) < length(vars)) stop("Variable names repeated")
-}
-
-# Merge together a series of data.frames
-#
-# Order of data frames should be from most complete to least complete 
-#
-# @arguments list of data frames to merge
-# @seealso \code{\link{merge_recurse}}
-# @keyword manip
-merge_all <- function(dfs, ...) {
-	if (length(dfs)==1) return(dfs[[1]])
-	df <- merge_recurse(dfs, ...)
-	df <- df[, match(names(dfs[[1]]), names(df))]
-	df[do.call("order", df[, -ncol(df), drop=FALSE]), ,drop=FALSE]
-}
-
-# Recursively merge data frames
-#
-# @arguments list of data frames to merge
-# @seealso \code{\link{merge_all}}
-# @keyword internal
-merge_recurse <- function(dfs, ...) {
-	if (length(dfs) == 2) {
-		merge(dfs[[1]], dfs[[2]], all=TRUE, sort=FALSE, ...)
-	} else {
-		merge(dfs[[1]], Recall(dfs[-1]), all=TRUE, sort=FALSE, ...)
+# @arguments data frame
+# @arguments margins to compute
+# @arguments aggregation function
+# @arguments other argument passed to aggregation function
+# @keyword internal 
+compute.margins <- function(data, margins, fun.aggregate, ..., df=FALSE) {
+	if (length(margins) == 0) return(data.frame())
+	
+	if (missing(fun.aggregate)) {
+		warning("Margins require fun.aggregate: length used as default", call.=FALSE)
+		fun.aggregate <- length
 	}
-}
\ No newline at end of file
+	exp <- function(x) {
+		if (df) {
+			condense.df(data, x, fun.aggregate, ...)
+		} else {
+			expand(condense(data, x, fun.aggregate, ...))
+		}
+	}
+	
+	do.call("rbind.fill",lapply(margins, exp))
+}
+
+
+# Margin variables
+# Works out list of variables to margin over to get desired margins.
+#
+# Variables that can't be margined over are dropped silently.
+#
+# @arguments column variables
+# @arguments row variables
+# @arguments vector of variable names to margin over.
+# @keyword internal
+margin.vars <- function(vars = list(NULL, NULL), margins = NULL) {
+	rows <- vars[[1]]
+	cols <- vars[[2]]
+	if (missing(margins) || is.null(margins)) return(NULL)
+	
+	# Nothing to margin over for last variable in column or row
+	row.margins <- intersect(rows[-length(rows)], margins)
+	col.margins <- intersect(cols[-length(cols)], margins)
+
+	grand.row <- "grand_row" %in% margins
+	grand.col <- "grand_col" %in% margins
+	
+	margin.intersect <- function(cols, col.margins, rows, row.margins) {
+		unlist(lapply(col.margins, function(col) {
+			c(lapply(row.margins, c, col), list(c(col, rows)))
+		}), recursive = FALSE)
+	}
+	
+	margins.all <- c(
+		margin.intersect(cols, col.margins, rows, row.margins),  
+		margin.intersect(rows, row.margins, cols, col.margins)
+	)
+
+	if (grand.row && !is.null(rows)) margins.all <- c(margins.all, list(cols), list(col.margins))
+	if (grand.col && !is.null(cols)) margins.all <- c(margins.all, list(rows), list(row.margins))
+	if (grand.col && grand.row  && !is.null(rows)  && !is.null(cols)) margins.all <- c(margins.all, list(numeric(0)))
+	
+	duplicates <- duplicated(lapply(lapply(margins.all,function(x) if(!is.null(x)) sort(x)), paste, collapse=""))
+	
+	margins.all[!duplicates]
+}
+
+# Dimension names
+# Convenience method for extracting row and column names 
+# 
+# @arguments data frame
+# @arguments variables to use
+# @keyword internal
+dim.names <- function(data, vars) {
+	if (!is.null(vars) && length(vars) > 0) {
+		unique(data[,vars,drop=FALSE]) 
+	} else {
+		data.frame(value="value") # use fun.aggregate instead of "value"? 
+	}
+}
